package fmon.world

import java.io._

import fmon.Config
import fmon.draw.tile._

class Tileset(val groundTiles: IndexedSeq[AutoTile], val doodadTiles: IndexedSeq[AutoTile], val info: IndexedSeq[TileInfo]) {
  def groundInfo(index: Int) = info(index)
  def doodadInfo(index: Int) = info(index - groundTiles.size)
}

case class TileInfo(
    val doesPassNorth: Boolean,
    val doesPassEast: Boolean,
    val doesPassSouth: Boolean,
    val doesPassWest: Boolean,
    val isCounter: Boolean) {
  
}

case class TilesetToken(
    val name: String,
    val a1: String,
    val a2: String,
    val a3: String,
    val a4: String,
    val a5: String,
    val b: String,
    val c: String,
    val info: IndexedSeq[TileInfo]) {
  
  def load(dir: File): Tileset = {
    val a1Tiles = AutoTilePalette.a2(new File(dir, a1), Config.tileSize).tiles
    val a2Tiles = AutoTilePalette.a2(new File(dir, a2), Config.tileSize).tiles
    val a3Tiles = AutoTilePalette.a3(new File(dir, a3), Config.tileSize).tiles
    val a4Tiles = AutoTilePalette.a4(new File(dir, a4), Config.tileSize).tiles
    val a5Tiles = AutoTilePalette.basic(new File(dir, a5), Config.tileSize).tiles
    val groundTiles = a1Tiles ++ a2Tiles ++ a3Tiles ++ a4Tiles ++ a5Tiles
    val bTiles = AutoTilePalette.partitioned(new File(dir, b), 2, Config.tileSize).tiles
    val cTiles = AutoTilePalette.partitioned(new File(dir, c), 2, Config.tileSize).tiles
    val doodadTiles = bTiles ++ cTiles
    new Tileset(groundTiles, doodadTiles, info)
  }
  
  override def toString = name
}