package fmon.world.ui

import java.io._

import javafx.application.Application
import javafx.fxml.FXML
import javafx.fxml.FXMLLoader
import javafx.fxml.JavaFXBuilderFactory
import javafx.scene.{control => jfxsc, layout => jfxsl, Parent, Scene}
import javafx.scene.input.KeyEvent
import javafx.stage.Stage

import scalafx.Includes._
import scalafx.animation.AnimationTimer
import scalafx.beans.property._
import scalafx.collections.ObservableBuffer
import scalafx.scene.control._
import scalafx.scene.image.ImageView
import scalafx.scene.input.{KeyCode, MouseEvent}
import scalafx.scene.paint.Color
import scalafx.scene.layout._
import scalafx.stage.FileChooser
import scalafx.stage.Modality
import scalafx.util.Duration
import FileChooser.ExtensionFilter

import fmon._
import fmon.draw._
import fmon.util.Direction
import fmon.world._
import fmon.util.YamlHelper
import Direction._

class GameView {
  @FXML var scroller: jfxsc.ScrollPane = _
  @FXML var background: jfxsl.TilePane = _
  @FXML var doodads: jfxsl.TilePane = _
  @FXML var characterPane: jfxsl.AnchorPane = _
  
  var hero: Actor = _
  var heroImg: AnimatedImageView = _
  var currDir: Option[Direction.Value] = None
  
  var lastTime = 0L
  val timer = AnimationTimer(t => {
    if (lastTime != 0L) {
      act(new Duration((t - lastTime) * 0.000001))
    }
    lastTime = t
  })
  
  def loadLevel(level: GameMap): Unit = {
    background.prefColumns = level.width
    val tiles = for (y <- 0 until level.height; x <- 0 until level.width) yield {
        new ImageView(level.smoothed(x, y)).delegate
    }
    
    background.children ++= tiles
    
    doodads.prefColumns = level.width
    val dtiles = for (y <- 0 until level.height; x <- 0 until level.width) yield {
        new ImageView(level.doodad(x, y).icon.croppedImage()).delegate
    }
    doodads.children ++= dtiles
    
    val heroFile = raw"C:\Users\James\Documents\Design\Project\Progena\Resources\characters\Actor1.png"
    hero = new Actor(CharSet(new File(heroFile), 1), 5, 4)
    
    heroImg = new AnimatedImageView(hero.sprite, new Duration(600)) {
      x = hero.x * Config.tileSize
      y = hero.y * Config.tileSize + Config.yOffset
    }
    characterPane.children += heroImg
    timer.start()
  }
  
  def onKeyDown(k: KeyEvent): Unit = {
    k.code match {
      case KeyCode.W | KeyCode.Up => setDir(North)
      case KeyCode.D | KeyCode.Right => setDir(East)
      case KeyCode.A | KeyCode.Left => setDir(West)
      case KeyCode.S | KeyCode.Down => setDir(South)
      case _ => ()
    }
  }
  
  def onKeyUp(k: KeyEvent): Unit = {
    k.code match {
      case KeyCode.W | KeyCode.Up => clearDir(North)
      case KeyCode.D | KeyCode.Right => clearDir(East)
      case KeyCode.A | KeyCode.Left => clearDir(West)
      case KeyCode.S | KeyCode.Down => clearDir(South)
      case _ => ()
    }
  }
  
  def setDir(dir: Direction.Value): Unit = {
    hero.pose = dir.toString()
    currDir = Some(dir)
    heroImg.play()
  }
  
  def clearDir(dir: Direction.Value): Unit = {
    if (currDir == Some(dir)) {
      currDir = None
      heroImg.stop()
    }
  }
  
  def act(dur: Duration): Unit = {
    currDir match {
      case Some(North) => hero.y -= dur.toSeconds() * Config.moveSpeed
      case Some(East) => hero.x += dur.toSeconds() * Config.moveSpeed
      case Some(West) => hero.x -= dur.toSeconds() * Config.moveSpeed
      case Some(South) => hero.y += dur.toSeconds() * Config.moveSpeed
      case _ => ()
    }
    heroImg.x = hero.x * Config.tileSize
    heroImg.y = hero.y * Config.tileSize
  }
}

object GameView {
  class GameApp extends Application {
    override def start(primaryStage: Stage): Unit = {
      val frameLoader = new FXMLLoader(getClass.getResource("GameView.fxml"))
      val root: Parent = frameLoader.load()
      val controller = frameLoader.getController[GameView]()
      
      val stageFile = raw"C:\Users\James\Documents\Design\Project\Progena\Data\outside.map"
      val level = GameMap.loadFrom(new File(stageFile))
      controller.loadLevel(level)
      //val builderLoader = new FXMLLoader(getClass.getResource("ElementBuilder.fxml"))
      //val builder: Parent = builderLoader.load()
      //val builderController = builderLoader.getController[ElementBuilder]()
      
      //controller.pane.children = builder
      //controller.builder = builderController
      
      val scene: Scene = new Scene(root)
      primaryStage.setScene(scene)
      primaryStage.width = 1280
      primaryStage.height = 800
      //primaryStage.maximized = true
      primaryStage.show()
    }
  }
  
  def main(args: Array[String]): Unit = {
    Application.launch(classOf[GameApp], args: _*)
  }
}